@@ -1,12 +1,7 @@
-gSquare = function(target, dataset, xIndex, csIndex, dataInfo=NULL, univariateModels=NULL, hash = FALSE, stat_hash=NULL, pvalue_hash=NULL)
+gSquare = function(target, dataset, xIndex, csIndex, dataInfo=NULL, univariateModels=NULL, hash = FALSE, 
+stat_hash=NULL, pvalue_hash=NULL,robust=FALSE)
 {
   #Conditional Independence test based on the G test of independence (log likelihood ratio  test)
-  #Suggested package in order to use this test: pcalg
-  
-  if(requireNamespace("pcalg", quietly = TRUE, warn.conflicts = FALSE) == FALSE){
-    warning("You have to load the package 'pcalg' in order to use the gSquare test");
-    return(NULL);
-  }
   
   csIndex[which(is.na(csIndex))] = 0;
   
@@ -35,8 +30,8 @@ gSquare = function(target, dataset, xIndex, csIndex, dataInfo=NULL, univariateMo
     {
       if(hash == TRUE)#update hash objects
       {
-        stat_hash$key <- 0;#.set(stat_hash , key , 0)
-        pvalue_hash$key <- 1;#.set(pvalue_hash , key , 1)
+        stat_hash[[key]] <- 0;#.set(stat_hash , key , 0)
+        pvalue_hash[[key]] <- 1;#.set(pvalue_hash , key , 1)
       }
       pvalue = 1;
       stat = 0;
@@ -87,8 +82,8 @@ gSquare = function(target, dataset, xIndex, csIndex, dataInfo=NULL, univariateMo
   
   if(hash == TRUE)
   {
-    stat_hash$key <- stat;#.set(stat_hash , key , stat)
-    pvalue_hash$key <- pvalue;#.set(pvalue_hash , key , pvalue)
+    stat_hash[[key]] <- stat;#.set(stat_hash , key , stat)
+    pvalue_hash[[key]] <- pvalue;#.set(pvalue_hash , key , pvalue)
   }
   
   results <- list(pvalue = pvalue, stat = stat, flag = flag, stat_hash=stat_hash, pvalue_hash=pvalue_hash);
