@@ -1,5 +1,6 @@
-testIndFisher = function(target, dataset, xIndex, csIndex, dataInfo=NULL, univariateModels=NULL , hash = FALSE, stat_hash=NULL, pvalue_hash=NULL)
-{
+testIndFisher = function(target, dataset, xIndex, csIndex, dataInfo=NULL, univariateModels=NULL , hash = FALSE, stat_hash=NULL,
+ pvalue_hash=NULL, robust=FALSE) 
+  {
   # TESTINDFISHER Fisher Conditional Independence Test for continous class variables
   # PVALUE = TESTINDFISHER(Y, DATA, XINDEX, CSINDEX, DATAINFO)
   # This test provides a p-value PVALUE for the NULL hypothesis H0 which is
@@ -7,10 +8,11 @@ testIndFisher = function(target, dataset, xIndex, csIndex, dataInfo=NULL, univar
   # Fisher's method (see reference below)
   
   # This method requires the following inputs
-  #   TARGET: a numeric vector containing the values of the target variable.
-  #   DATASET: a numeric data matrix containing the variables for performing the test.
-  #   XINDEX: the index of the variable whose association with the target we want to test.
-  #   CSINDEX: the indices if the variable to condition on.
+  #   TARGET: a numeric vector containing the values of the target (continuous) variable. 
+  #   Its support can be R or any number betweeen 0 and 1, i.e. it contains proportions.
+  #   DATASET: a numeric data matrix containing the variables for performing the test. They can be only be continuous variables. 
+  #   XINDEX: the index of the variable whose association with the target we want to test. 
+  #   CSINDEX: the indices if the variable to condition on. 
   #   DATAINFO: information on the structure of the data
   
   # this method returns: the pvalue PVALUE, the statistic STAT and a control variable FLAG.
@@ -33,7 +35,11 @@ testIndFisher = function(target, dataset, xIndex, csIndex, dataInfo=NULL, univar
   pvalue = 1;
   stat = 0;
   flag = 0;
-  
+   if ( all(target>0 & target<1) ) ## are they proportions?
+   { 
+     target = log( target/(1-target) ) 
+   }
+  n = length( target )
   csIndex[which(is.na(csIndex))] = 0;
   
   if(hash == TRUE)
@@ -59,8 +65,8 @@ testIndFisher = function(target, dataset, xIndex, csIndex, dataInfo=NULL, univar
   {
     if(hash == TRUE)#update hash objects
     {
-      stat_hash$key <- 0;#.set(stat_hash , key , 0)
-      pvalue_hash$key <- 1;#.set(pvalue_hash , key , 1)
+      stat_hash[[key]] <- 0;#.set(stat_hash , key , 0)
+      pvalue_hash[[key]] <- 1;#.set(pvalue_hash , key , 1)
     }
     results <- list(pvalue = 1, stat = 0, flag = 1 , stat_hash=stat_hash, pvalue_hash=pvalue_hash);
     return(results);
@@ -112,8 +118,8 @@ testIndFisher = function(target, dataset, xIndex, csIndex, dataInfo=NULL, univar
       {
         if(hash == TRUE)#update hash objects
         {
-          stat_hash$key <- 0;#.set(stat_hash , key , 0)
-          pvalue_hash$key <- 1;#.set(pvalue_hash , key , 1)
+          stat_hash[[key]] <- 0;#.set(stat_hash , key , 0)
+          pvalue_hash[[key]] <- 1;#.set(pvalue_hash , key , 1)
         }
         results <- list(pvalue = 1, stat = 0, flag = 1 , stat_hash=stat_hash, pvalue_hash=pvalue_hash);
         return(results);
@@ -125,8 +131,8 @@ testIndFisher = function(target, dataset, xIndex, csIndex, dataInfo=NULL, univar
         {
           if(hash == TRUE)#update hash objects
           {
-            stat_hash$key <- 0;#.set(stat_hash , key , 0)
-            pvalue_hash$key <- 1;#.set(pvalue_hash , key , 1)
+            stat_hash[[key]] <- 0;#.set(stat_hash , key , 0)
+            pvalue_hash[[key]] <- 1;#.set(pvalue_hash , key , 1)
           }
           results <- list(pvalue = 1, stat = 0, flag = 1 , stat_hash=stat_hash, pvalue_hash=pvalue_hash);
           return(results);
@@ -140,8 +146,8 @@ testIndFisher = function(target, dataset, xIndex, csIndex, dataInfo=NULL, univar
   {
     if(hash == TRUE)#update hash objects
     {
-      stat_hash$key <- 0;#.set(stat_hash , key , 0)
-      pvalue_hash$key <- 1;#.set(pvalue_hash , key , 1)
+      stat_hash[[key]] <- 0;#.set(stat_hash , key , 0)
+      pvalue_hash[[key]] <- 1;#.set(pvalue_hash , key , 1)
     }
     results <- list(pvalue = 1, stat = 0, flag = 1 , stat_hash=stat_hash, pvalue_hash=pvalue_hash);
     return(results);
@@ -151,8 +157,8 @@ testIndFisher = function(target, dataset, xIndex, csIndex, dataInfo=NULL, univar
   cs = as.matrix(cs)
   cs = cs[,apply(cs, 2, var, na.rm=TRUE) != 0]
   
-  #trycatch for dealing with errors
-  res <- tryCatch(
+#trycatch for dealing with errors
+res <- tryCatch(
 {
   #if the conditioning set (cs) is empty, we use a simplified formula
   if(length(cs) == 0)
@@ -165,35 +171,44 @@ testIndFisher = function(target, dataset, xIndex, csIndex, dataInfo=NULL, univar
       results <- list(pvalue = pvalue, stat = stat, flag = flag , stat_hash=stat_hash, pvalue_hash=pvalue_hash);
       return(results);
     }
-    
     #compute the correlation coefficient between x,target directly
-    stat = abs(cor(x,target));
+    if (robust == TRUE) { ## robust correlation
+      b1 = coef(MASS::rlm(target ~ x, maxit=1000))[2]
+      b2 = coef(MASS::rlm(x ~ target, maxit=1000))[2]
+      stat = sqrt( abs(b1*b2) )
+    }else{
+      stat = abs(cor(x, target));
+    }
   }else{
     #perform the test with the cs
-    tmpm = cbind(x,target,cs);
     
-    corrMatrix = cor(tmpm);
-    
-    xyIdx = 1:2;
-    csIdx = 3:(ncol(as.matrix(cs))+2); #or csIdx = 3;
-    
-    #use pseudoinverse instead of solve (if there is an error or in general)
-    residCorrMatrix = (corrMatrix[xyIdx, xyIdx]) - as.matrix(corrMatrix[xyIdx, csIdx])%*%(solve( as.matrix(corrMatrix[csIdx, csIdx]) , rbind(corrMatrix[csIdx, xyIdx])));
-    #residCorrMatrix = (corrMatrix[xyIdx, xyIdx]) - as.matrix(corrMatrix[xyIdx, csIdx])%*%(mldivide(as.matrix(corrMatrix[csIdx, csIdx]),rbind(corrMatrix[csIdx, xyIdx]) ));
-    
-    stat = abs(residCorrMatrix[1,2] / sqrt(residCorrMatrix[1,1] * residCorrMatrix[2,2]));
+   if (robust == TRUE) { ## robust correlation
+     e1 = resid( MASS::rlm(target ~ dataset[, csIndex], maxit=1000) ) 
+     e2 = resid( MASS::rlm(dataset[, xIndex] ~ dataset[, csIndex], maxit=1000) ) 
+     stat = cor(e1,e2) 
+   }else{
+     tmpm = cbind(x,target,cs);
+     
+     corrMatrix = cor(tmpm);
+     
+     xyIdx = 1:2;
+     csIdx = 3:(ncol(as.matrix(cs))+2); #or csIdx = 3;
+     
+     residCorrMatrix = (corrMatrix[xyIdx, xyIdx]) - as.matrix(corrMatrix[xyIdx, csIdx])%*%(solve( as.matrix(corrMatrix[csIdx, csIdx]) , rbind(corrMatrix[csIdx, xyIdx])) );
+
+     stat = abs(residCorrMatrix[1,2] / sqrt(residCorrMatrix[1,1] * residCorrMatrix[2,2]));
+   }
   }
-  
   #lets calculate the p-value
   
   #comparing against the Student's t distribution
-  z = 0.5*log((1+stat)/(1-stat));
-  df = length(x) - ncol(as.matrix(cs)) - 3; #degrees of freedom
-  w = sqrt(df)*z;
+  z = 0.5*log( (1+stat)/(1-stat) );
+  df = n - ncol(as.matrix(cs)) - 3; #degrees of freedom
+  w = sqrt(df) * z;
   
-  pvalue = 2 * pt(-abs(w),df);#?dt for documentation
+  pvalue = 2 * pt(-abs(w), df) ;  # ?dt for documentation
   #or alternatively we can calculate the p-value by comparing against the normal distribution
-  #pvalue = 2 * pnorm(-abs(w));
+  #pvalue = 2 * pnorm(-abs(w));  ### 
   
   flag = 1;
   
@@ -202,13 +217,13 @@ testIndFisher = function(target, dataset, xIndex, csIndex, dataInfo=NULL, univar
   {
     pvalue = 1;
     stat = 0;
-    flag = 0;
+    flag = 1;
   }else{
     #update hash objects
     if(hash == TRUE)
     {
-      stat_hash$key <- stat;#.set(stat_hash , key , stat)
-      pvalue_hash$key <- pvalue;#.set(pvalue_hash , key , pvalue)
+      stat_hash[[key]] <- stat;#.set(stat_hash , key , stat)
+      pvalue_hash[[key]] <- pvalue;#.set(pvalue_hash , key , pvalue)
     }
   }
   
@@ -219,23 +234,23 @@ testIndFisher = function(target, dataset, xIndex, csIndex, dataInfo=NULL, univar
   
 },
 error=function(cond) {
-  #message(paste("error in try catch of the testIndFisher test"))
-  #message("Here's the original error message:")
-  #message(cond)
+#    message(paste("warning in try catch of the testIndFisher test"))
+#    message("Here's the original message:")
+#    message(cond)
+#   
+#   #        
+#   #        #for debug
+#           print("\nxIndex = \n");
+#           print(xIndex);
+#           print("\ncsindex = \n");
+#           print(csIndex);
+#   
+#   stop();
   
-  #        
-  #        #for debug
-  #        printf("\nxIndex = \n");
-  #        print(xIndex);
-  #        printf("\ncsindex = \n");
-  #        print(csIndex);
-  
-  #stop();
-  
-  #error case
+  #error case (we are pretty sure that the only error case is when x,cs are highly correlated and the inversion of the matrix is not possible)
   pvalue = 1;
   stat = 0;
-  flag = 0;
+  flag = 1;
   
   results <- list(pvalue = pvalue, stat = stat, flag = flag , stat_hash=stat_hash, pvalue_hash=pvalue_hash);
   return(results);
