@@ -1,4 +1,5 @@
-testIndLogistic = function(target, dataset, xIndex, csIndex, dataInfo=NULL , univariateModels=NULL , hash = FALSE,stat_hash=NULL, pvalue_hash=NULL, target_type=0)
+testIndLogistic = function(target, dataset, xIndex, csIndex, dataInfo=NULL , univariateModels=NULL ,
+ hash = FALSE,stat_hash=NULL, pvalue_hash=NULL, target_type=0, robust=FALSE)
 {
   #   TESTINDLOGISTIC Conditional Independence Test based on logistic regression for binary,categorical or ordinal class variables
   #
@@ -14,12 +15,12 @@ testIndLogistic = function(target, dataset, xIndex, csIndex, dataInfo=NULL , uni
   #   the number of categories
   #
   #   dataset: a numeric data matrix containing the variables for performing
-  #   the conditional independence test
+  #   the conditional independence test. They can be mixed variables, either continous or categorical
   #
   #   xIndex: the index of the variable whose association with the target
-  #   must be tested.
+  #   must be tested. Can be any type of variable, either continous or categorical.
   #
-  #   csIndex: the indices if the variables to condition on.
+  #   csIndex: the indices of the variables to condition on. They can be mixed variables, either continous or categorical
   #
   #   target_Type: the type of the target
   #   target_type == 1 (binomial target)
@@ -56,10 +57,9 @@ testIndLogistic = function(target, dataset, xIndex, csIndex, dataInfo=NULL , uni
   #initialization
   
   #cast factor into numeric vector
-  #target = as.numeric(as.vector(target));
-  if(class(target) == "factor")
+  if(is.ordered(target) == FALSE)
   {
-    target = as.numeric(target)-1;
+    target = as.factor(as.numeric(as.vector(target)));
   }
   
   csIndex[which(is.na(csIndex))] = 0
@@ -92,8 +92,8 @@ testIndLogistic = function(target, dataset, xIndex, csIndex, dataInfo=NULL , uni
   {
     if(hash == TRUE)#update hash objects
     {
-      stat_hash$key <- 0;#.set(stat_hash , key , 0)
-      pvalue_hash$key <- 1;#.set(pvalue_hash , key , 1)
+      stat_hash[[key]] <- 0;#.set(stat_hash , key , 0)
+      pvalue_hash[[key]] <- 1;#.set(pvalue_hash , key , 1)
     }
     results <- list(pvalue = 1, stat = 0, flag = 1, stat_hash=stat_hash, pvalue_hash=pvalue_hash);
     return(results);
@@ -125,8 +125,8 @@ testIndLogistic = function(target, dataset, xIndex, csIndex, dataInfo=NULL , uni
   {
     if(hash == TRUE)#update hash objects
     {
-      stat_hash$key <- 0;#.set(stat_hash , key , 0)
-      pvalue_hash$key <- 1;#.set(pvalue_hash , key , 1)
+      stat_hash[[key]] <- 0;#.set(stat_hash , key , 0)
+      pvalue_hash[[key]] <- 1;#.set(pvalue_hash , key , 1)
     }
     results <- list(pvalue = 1, stat = 0, flag = 1, stat_hash=stat_hash, pvalue_hash=pvalue_hash);
     return(results);
@@ -154,8 +154,8 @@ testIndLogistic = function(target, dataset, xIndex, csIndex, dataInfo=NULL , uni
       {
         if(hash == TRUE)#update hash objects
         {
-          stat_hash$key <- 0;#.set(stat_hash , key , 0)
-          pvalue_hash$key <- 1;#.set(pvalue_hash , key , 1)
+          stat_hash[[key]] <- 0;#.set(stat_hash , key , 0)
+          pvalue_hash[[key]] <- 1;#.set(pvalue_hash , key , 1)
         }
         results <- list(pvalue = 1, stat = 0, flag = 1 , stat_hash=stat_hash, pvalue_hash=pvalue_hash);
         return(results);
@@ -167,8 +167,8 @@ testIndLogistic = function(target, dataset, xIndex, csIndex, dataInfo=NULL , uni
         {
           if(hash == TRUE)#update hash objects
           {
-            stat_hash$key <- 0;#.set(stat_hash , key , 0)
-            pvalue_hash$key <- 1;#.set(pvalue_hash , key , 1)
+            stat_hash[[key]] <- 0;#.set(stat_hash , key , 0)
+            pvalue_hash[[key]] <- 1;#.set(pvalue_hash , key , 1)
           }
           results <- list(pvalue = 1, stat = 0, flag = 1 , stat_hash=stat_hash, pvalue_hash=pvalue_hash);
           return(results);
@@ -182,12 +182,16 @@ testIndLogistic = function(target, dataset, xIndex, csIndex, dataInfo=NULL , uni
     target_type = dataInfo$target_type;
     if(dataInfo$target_type == "nominal")
     {
+      #cat("Multinomial Logistic Regression")
       target_type = 2;
     }else if(dataInfo$target_type == "ordinal"){
+      #cat("Ordinal Logistic Regression")
       target_type = 3;
     }else if(dataInfo$target_type == "binary"){
+      #cat("Binary Logistic Regression")
       target_type = 1;
     }else{
+      #cat("Multinomial Logistic Regression")
       target_type = 2; #default value in case of bad definition
     }
   }else{
@@ -207,7 +211,7 @@ testIndLogistic = function(target, dataset, xIndex, csIndex, dataInfo=NULL , uni
     results <- list(pvalue = pvalue, stat = stat, flag = flag, stat_hash=stat_hash, pvalue_hash=pvalue_hash);
     return(results);
   }
-  
+  n=length(target)  ## sample size
   #trycatch for dealing with errors
   res <- tryCatch(
 {
@@ -216,11 +220,11 @@ testIndLogistic = function(target, dataset, xIndex, csIndex, dataInfo=NULL , uni
   if(yCounts == 2)
   {
     target_type = 1;
-  }else{
+  }#else{
     #nominal or ordinal target
     #correct encoding for target
-    target = target + 1;
-  }
+    #target = target + 1;
+  #}
   
   #if the conditioning set (cs) is empty, we use the t-test on the coefficient of x.
   if(length(cs) == 0)
@@ -239,61 +243,100 @@ testIndLogistic = function(target, dataset, xIndex, csIndex, dataInfo=NULL , uni
     if(target_type == 1) #binomial
     {
       #Fitting generalized Linear Models
-      dev1 = glm(target~t(t(rep(1,dim(dataset)[1]))) , family = binomial())$deviance;
-      dev2 = glm(target~x , family = binomial())$deviance;
-    }
-    else if(target_type == 2) #nominal-categorical
+      ff <- target ~ x
+      m <- model.frame(ff)
+      mat <- model.matrix(ff, m)
+      mat <- mat[1:n, ]
+      fit2 = glm.fit(mat, target, family=binomial(logit))
+      dev1 = fit2$null.deviance 
+      dev2 = fit2$deviance;
+      p1 = 1
+      p2 = length( coef(fit2) )
+    }else if(target_type == 2) #nominal-categorical
     {
       #Fitting multinomial Logistic regression
-      #dev1 = multinom(as.factor(target)~rep(0,dim(dataset)[1]) , as.data.frame(dataset) ,maxit=100 , trace=FALSE)$deviance;
-      #dev2 = multinom(as.factor(target)~x , as.data.frame(dataset) ,  maxit=100 , trace=FALSE)$deviance; #not the exact dev as i expect (matlabs mnrfit) ~-1
-      
-      fit <- VGAM::vglm(target ~ 1, VGAM::multinomial, as.data.frame(dataset))
-      dev1 = deviance(fit)
-      fit <- VGAM::vglm(target ~ x, VGAM::multinomial, as.data.frame(dataset))
-      dev2 = deviance(fit)
+      fit1 <- nnet::multinom(target ~ 1, trace = F)
+      fit2 <- nnet::multinom(target ~ x, trace = F)
+      dev1 = fit1$deviance
+      dev2 = fit2$deviance
+      p1 = yCounts - 1
+      p2 = length( coef(fit2) )
+  
     }
     else if(target_type == 3) #ordinal
     {
       #Fitting ordinal Logistic regression
-      dev1 = MASS::polr(as.factor(target)~rep(0,dim(dataset)[1]) , dataset)$deviance;
-      dev2 = MASS::polr(as.factor(target)~x , dataset)$deviance;
+      fit1 <- ordinal::clm(target ~ 1 )
+      fit2 <- ordinal::clm(target ~ x )
+      dev1 = 2* fit1$logLik
+      dev2 = 2* fit2$logLik
+      p1 = yCounts - 1
+      p2 = length( coef(fit2) )
+      
     }
   }else{
     if(target_type == 1) #binomial
     {
       #Fitting generalized Linear Models
-      dev1 = glm(target~as.matrix(cs) , family = binomial())$deviance;
-      dev2 = glm(target~x+as.matrix(cs) , family = binomial())$deviance;
+      kapa = length(csIndex)
+      ff1 = as.formula(paste("target ~ ",paste("dataset[,",csIndex[1:kapa],"]",sep="",collapse="+"), sep=""))
+      m1 <- model.frame(ff1)
+      mat1 <- model.matrix(ff1, m1)
+      mat1 <- mat1[1:n,]
+      fit1 = glm.fit(mat1, target, family=binomial(logit))
+      dev1 = fit1$deviance 
+      p1 = length( coef(fit1) )      
+      
+      ff2 = as.formula(paste(paste("target ~ ",paste("dataset[,",csIndex[1:kapa],"]",sep="",collapse="+"), sep="") , "+dataset[,",xIndex,"]", sep = ""))
+      m2 <- model.frame(ff2)
+      mat2 <- model.matrix(ff2, m2)
+      mat2 <- mat2[1:n,]
+      fit2 = glm.fit(mat2, target, family=binomial(logit))
+      dev2 = fit2$deviance 
+      p2 = length( coef(fit2) )
+      
     }
     else if(target_type == 2) #nominal-categorical
     {
       #Fitting multinomial Logistic regression
-      #dev1 = multinom(as.factor(target)~as.matrix(cs) , as.data.frame(dataset) , maxit=100 , trace=FALSE)$deviance;
-      #dev2 = multinom(as.factor(target)~cbind(x,as.matrix(cs)) , as.data.frame(dataset) , maxit=100 , trace=FALSE)$deviance; #not the exact dev as i expect (matlabs mnrfit) ~-1
-      
-      fit <- VGAM::vglm(target ~ as.matrix(cs), VGAM::multinomial, as.data.frame(dataset))
-      dev1 = deviance(fit)
-      fit <- VGAM::vglm(target ~ x+as.matrix(cs), VGAM::multinomial, as.data.frame(dataset))
-      dev2 = deviance(fit)
+      if(length(csIndex) == 1){
+        fit1 = nnet::multinom( target ~ dataset[, csIndex], data = dataset[,c(csIndex, xIndex)], trace = F)
+      }else{
+        fit1 = nnet::multinom( target ~., data = dataset[, csIndex], trace = F)
+      }
+      dev1 = deviance(fit1)
+      fit2 <- nnet::multinom(target ~.,  data = dataset[, c(xIndex, csIndex)], trace = F)
+      dev2 = deviance(fit2)
+      p1 = length(coef(fit1))
+      p2 = length(coef(fit2))
     }
     else if(target_type == 3) #ordinal
     {
       #Fitting ordinal Logistic regression
-      dev1 = MASS::polr(as.factor(target)~cs , dataset)$deviance;
-      dev2 = MASS::polr(as.factor(target)~x+cs , dataset)$deviance;
+      if(length(csIndex) == 1){
+        fit1 = ordinal::clm( target ~ dataset[, csIndex], data = dataset[,c(csIndex, xIndex)])
+      }else{
+        fit1 = ordinal::clm( target ~., data = dataset[, csIndex] )
+      }
+      dev1 = 2*fit1$logLik
+      fit2 <- ordinal::clm(target ~., data = dataset[, c(csIndex, xIndex)])
+      dev2 = 2*fit2$logLik
+      p1 = length(coef(fit1))
+      p2 = length(coef(fit2))
     }
   }
   
   #calculate the p value and stat.
-  stat = abs(dev1 - dev2);
-  pvalue = 1 - pchisq(stat, yCounts-1);
+  stat = abs (dev1 - dev2) 
+  df = abs( p2 - p1 ) ## a bit stupid, but it works
+  pvalue = 1 - pchisq(stat, df); 
   flag = 1;
+  
   #update hash objects
   if(hash == TRUE)
   {
-    stat_hash$key <- stat;#.set(stat_hash , key , stat)
-    pvalue_hash$key <- pvalue;#.set(pvalue_hash , key , pvalue)
+    stat_hash[[key]] <- stat;#.set(stat_hash , key , stat)
+    pvalue_hash[[key]] <- pvalue;#.set(pvalue_hash , key , pvalue)
   }
   
   #last error check
@@ -309,22 +352,22 @@ testIndLogistic = function(target, dataset, xIndex, csIndex, dataInfo=NULL , uni
   
 },
 error=function(cond) {
-  #message(paste("error in try catch of the testIndLogistic test"))
-  #message("Here's the original error message:")
-  #message(cond)
-  
-  #for debug
-  #   printf("\nxIndex = \n");
-  #   print(xIndex);
-  #   printf("\ncsindex = \n");
-  #   print(csIndex);
+#   message(paste("error in try catch of the testIndLogistic test"))
+#   message("Here's the original error message:")
+#   message(cond)
+#   
+#   #for debug
+#     print("\nxIndex = \n");
+#     print(xIndex);
+#     print("\ncsindex = \n");
+#     print(csIndex);
   
   #error case
   pvalue = 1;
   stat = 0;
   flag = 0;
   
-  #stop();
+  # stop();
   
   results <- list(pvalue = pvalue, stat = stat, flag = flag, stat_hash=stat_hash, pvalue_hash=pvalue_hash);
   return(results);
